# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2020-03-27 18:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_project', '0008_auto_20200326_2132'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='status',
            field=models.CharField(choices=[('Public', 'Public'), ('Private', 'Private'), ('Archived', 'Archived')], default='Private', max_length=8),
        ),
    ]
